@@ -36,7 +36,10 @@ except:
     isHildon=False
 
 
-version="0.4"
+version="0.5"
+
+# The file header
+HEADER="#TrustMe 1"
 
 if not isHildon:
     class  hildon:
@@ -70,6 +73,15 @@ def errorPopup(message, toplevel=None):
     dialog.connect('response', lambda dialog, response: dialog.destroy())
     dialog.run()
 
+def infoPopup(message, toplevel=None):
+    dialog=gtk.MessageDialog(toplevel,
+                             gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
+                             gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
+                             message)
+    dialog.set_title('Note')
+    dialog.connect('response', lambda dialog, response: dialog.destroy())
+    dialog.run()
+
 
 
 def makePasswordEntry(returnCallback=None):
@@ -99,7 +111,13 @@ def returnKeyHandler(widget, event, func):
 def defaultFileName():
     appDir=os.getenv('HOME')+"/.trustme"
     return appDir+"/default.db"
-                
+
+
+def showBanner(toplevel, msg):
+    hildon.hildon_banner_show_information(toplevel,
+                                          None,
+                                          msg)
+
 
 class Panel():
 
@@ -135,7 +153,8 @@ class Panel():
         # Some menu entries are only enabled for certain panels.
         # Initially, I compared self.__class__.__name__ against
         # the relevant classes but decided that was to kludgy.
-        self.main.changePasswordItem.set_sensitive(self.canChangePassword)
+        for item in self.main.changePasswordItems:
+            item.set_sensitive(self.canChangePassword)
         
         self.prepare(arg)
 
@@ -582,6 +601,29 @@ class ChangePasswordPanel(Panel):
 
 class Main(hildon.Program):
 
+    
+    def queryFileName(self, title, openForRead):
+        if openForRead:
+            action=gtk.FILE_CHOOSER_ACTION_OPEN
+        else:
+            action=gtk.FILE_CHOOSER_ACTION_SAVE
+            
+        if isHildon:
+            dialog=hildon.FileChooserDialog(self.toplevel, action)
+        else:
+            dialog=gtk.FileChooserDialog(
+                title=title,
+                action=action,
+                buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK)
+                )
+        response=dialog.run()
+        fn=dialog.get_filename()
+        dialog.destroy()
+        if response==gtk.RESPONSE_CANCEL:
+            fn=None
+        return fn
+
+
     def aboutCb(self, data):
         dialog=gtk.AboutDialog()
         dialog.set_name('TrustMe')
@@ -597,6 +639,71 @@ class Main(hildon.Program):
         dialog.run()
         dialog.destroy()
 
+    def exportCb(self, data):
+        self.dialog=gtk.MessageDialog(parent=self.toplevel,
+                                      flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
+                                      buttons=gtk.BUTTONS_YES_NO,
+                                      message_format="Exporting is a security risk.\n\nDo you really want to export to an un-encrypted file?"
+                                     )
+        result=self.dialog.run()
+        self.dialog.destroy()
+        self.dialog=None
+        if result==gtk.RESPONSE_YES:
+            fn=self.queryFileName("Specify export file", False);
+            if fn:
+                self.clearAlarm()
+                try:
+                    content=self.store.toString()
+                    fp=open(fn, 'w')
+                    fp.write(HEADER+"\n")
+                    fp.write(content)
+                    fp.close()
+                    if isHildon:
+                        showBanner(self.toplevel, "Export Complete")
+                except IOError:
+                    errorPopup("Could not save file - I/O Error");
+
+
+    #
+    # Restore the database from a plain text file.
+    #
+    def importCb(self, data):
+        self.dialog=gtk.MessageDialog(parent=self.toplevel,
+                                      flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
+                                      buttons=gtk.BUTTONS_YES_NO,
+                                      message_format="Importing will overwrite your complete database.\n\nAre you sure you want to do this?"
+                                     )
+        result=self.dialog.run()
+        self.dialog.destroy()
+        self.dialog=None
+        if result==gtk.RESPONSE_YES:
+            fn=self.queryFileName("Specify file to import", True);
+            if fn:
+                self.clearAlarm()
+                try:
+                    fp=open(fn, 'r')
+                    header=fp.readline().strip()
+                    content=fp.read()
+                    fp.close()
+                    if header != HEADER:
+                        raise ValueError("Incorrect file header '"+header+"' detected")
+                    self.store.fromString(content)
+                    self.mainPanel.activate()
+                    if isHildon:
+                        showBanner(self.toplevel, "Import Complete")
+                except ValueError, err:
+                    errorPopup("Could not load file, reason: "+err.__str__());
+                except IOError, err:
+                    errorPopup("Could not load file: "+err.__str__());
+                else:
+                    # file was loaded successfully
+                    # Update the encrypted database
+                    self.vault.save(self.encryptedFileName, HEADER,content)
+                    infoPopup("Database was restored. You may want to delete the un-encrypted import file.");
+
+
+
+
     def key_press_cb(self, widget, event, *args):
         # Fullscreen key
         if event.keyval==gtk.keysyms.F6:
@@ -673,10 +780,8 @@ class Main(hildon.Program):
     def save(self):
         # turn alarm off - don't want to be interrupted while saving
         self.clearAlarm()
-
-        header="#TrustMe 1"
         content=self.store.toString()
-        self.vault.save(self.encryptedFileName, header,content)
+        self.vault.save(self.encryptedFileName, HEADER,content)
         
 
 
@@ -712,11 +817,21 @@ class Main(hildon.Program):
             helpMenu=gtk.Menu()
             item.set_submenu(helpMenu)
 
+        self.changePasswordItems=[];
         item=gtk.MenuItem("Change Password")
         item.connect_object("activate", lambda self, dummy=None: self.changePasswordPanel.activate(), self)
         fileMenu.append(item)
         # remember this item since it'll be disabled by some of the panels
-        self.changePasswordItem=item
+        self.changePasswordItems.append(item);
+
+        item=gtk.MenuItem("Export")
+        item.connect_object("activate", self.exportCb, None)
+        fileMenu.append(item)
+        self.changePasswordItems.append(item);
+        item=gtk.MenuItem("Import")
+        item.connect_object("activate", self.importCb, None)
+        fileMenu.append(item)
+        self.changePasswordItems.append(item);
 
         item=gtk.MenuItem("About")
         item.connect_object("activate", self.aboutCb, None)
